@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.800';
+our $VERSION = '1.801';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -86,10 +86,6 @@ sub import {
    return;
 }
 
-END {
-   %{ $_MCE } = ();
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Init and finish routines.
@@ -113,21 +109,22 @@ sub finish (@) {
    shift if (defined $_[0] && $_[0] eq 'MCE::Flow');
    my $_pkg = (defined $_[0]) ? shift : "$$.$_tid.".caller();
 
-   if ( $_pkg eq 'MCE::Shared::Server' ) {
-      MCE::Flow->finish($_, 1) for ( keys %{ $_MCE } );
-      %{ $_MCE } = ();
+   if ( $_pkg eq 'MCE' ) {
+      for my $_k ( keys %{ $_MCE } ) { MCE::Flow->finish($_k, 1); }
    }
-   elsif ( exists $_MCE->{$_pkg} ) {
-      MCE::_save_state(), $_MCE->{$_pkg}->shutdown(@_), MCE::_restore_state()
-         if $_MCE->{$_pkg}{_spawned};
+   elsif ( $_MCE->{$_pkg} && $_MCE->{$_pkg}{_init_pid} eq "$$.$_tid" ) {
+      $_MCE->{$_pkg}->shutdown(@_) if $_MCE->{$_pkg}{_spawned};
 
       delete $_user_tasks->{$_pkg};
       delete $_prev_c->{$_pkg};
       delete $_prev_n->{$_pkg};
       delete $_prev_t->{$_pkg};
       delete $_prev_w->{$_pkg};
+      delete $_MCE->{$_pkg};
    }
 
+   @_ = ();
+
    return;
 }
 
@@ -351,7 +348,10 @@ sub run (@) {
    if ($_init_mce) {
       $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
 
-      _gen_user_tasks($_pid, \@_code, \@_name, \@_thrs, \@_wrks);
+      ## must clear arrays for nested session to work with Perl < v5.14
+      _gen_user_tasks($_pid, [@_code], [@_name], [@_thrs], [@_wrks]);
+
+      @_code = @_name = @_thrs = @_wrks = ();
 
       my %_opts = (
          max_workers => $_max_workers, task_name => $_tag,
@@ -428,13 +428,13 @@ sub run (@) {
       }
    }
 
-   delete $_MCE->{$_pid}{gather} if (defined $_wa);
+   MCE::_restore_state();
 
-   if ($^S || $ENV{'PERL_IPERL_RUNNING'} || $INC{'MCE/Hobo.pm'}) {
+   if ($^S || $ENV{'PERL_IPERL_RUNNING'}) {
       $_MCE->{$_pid}->shutdown(); # shutdown if in eval state
    }
 
-   MCE::_restore_state();
+   delete $_MCE->{$_pid}{gather} if (defined $_wa);
 
    return ((defined $_wa) ? @_a : ());
 }
@@ -499,7 +499,7 @@ MCE::Flow - Parallel flow model for building creative applications
 
 =head1 VERSION
 
-This document describes MCE::Flow version 1.800
+This document describes MCE::Flow version 1.801
 
 =head1 DESCRIPTION
 
