@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.800';
+our $VERSION = '1.801';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -227,7 +227,7 @@ sub new {
       MCE::Util::_sock_pair($_Q, qw(_ar_sock _aw_sock)) if $_Q->{_await};
 
       if (exists $_argv{queue} && scalar @{ $_argv{queue} }) {
-         1 until syswrite $_Q->{_qw_sock}, $LF;
+         syswrite $_Q->{_qw_sock}, $LF;
       }
    }
 
@@ -654,11 +654,13 @@ sub _heap_insert_high {
          $_Q->{_tsem} = $_t;
 
          if ($_Q->_pending() <= $_t) {
-            1 until syswrite $_Q->{_aw_sock}, $LF;
+            syswrite $_Q->{_aw_sock}, $LF;
          } else {
             $_Q->{_asem} += 1;
          }
 
+         print {$_DAU_R_SOCK} $LF;
+
          return;
       },
 
@@ -669,7 +671,7 @@ sub _heap_insert_high {
          $_Q = $_all->{$_id};
 
          if ($_Q->_has_data()) {
-            1 until sysread $_Q->{_qr_sock}, my($_buf), 1;
+            sysread $_Q->{_qr_sock}, my($_buf), 1;
          }
          $_Q->_clear();
 
@@ -695,7 +697,7 @@ sub _heap_insert_high {
          }
          else {
             if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-               1 until syswrite $_Q->{_qw_sock}, $LF;
+               syswrite $_Q->{_qw_sock}, $LF;
             }
             push @{ $_Q->{_datq} }, @{ $_MCE->{thaw}($_buf) };
          }
@@ -714,7 +716,7 @@ sub _heap_insert_high {
          $_Q = $_all->{$_id};
 
          if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-            1 until syswrite $_Q->{_qw_sock}, $LF;
+            syswrite $_Q->{_qw_sock}, $LF;
          }
          $_Q->_enqueuep($_p, @{ $_MCE->{thaw}($_buf) });
 
@@ -738,7 +740,7 @@ sub _heap_insert_high {
          }
          else {
             if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-               1 until syswrite $_Q->{_qw_sock}, $LF;
+               syswrite $_Q->{_qw_sock}, $LF;
             }
             push @{ $_Q->{_datq} }, $_;
          }
@@ -757,7 +759,7 @@ sub _heap_insert_high {
          $_Q = $_all->{$_id};
 
          if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-            1 until syswrite $_Q->{_qw_sock}, $LF;
+            syswrite $_Q->{_qw_sock}, $LF;
          }
          $_Q->_enqueuep($_p, $_buf);
 
@@ -790,7 +792,7 @@ sub _heap_insert_high {
                $_pending = int($_pending / $_cnt) if ($_cnt);
                if ($_pending) {
                   $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
-                  for (1 .. $_pending) { 1 until syswrite $_Q->{_qw_sock}, $LF }
+                  for (1 .. $_pending) { syswrite $_Q->{_qw_sock}, $LF }
                }
                $_Q->{_dsem} = $_pending;
             }
@@ -800,7 +802,7 @@ sub _heap_insert_high {
          }
          else {
             ## Otherwise, never to exceed one byte in the channel
-            if ($_Q->_has_data()) { 1 until syswrite $_Q->{_qw_sock}, $LF }
+            if ($_Q->_has_data()) { syswrite $_Q->{_qw_sock}, $LF }
          }
 
          if ($_cnt) {
@@ -826,7 +828,7 @@ sub _heap_insert_high {
          }
 
          if ($_Q->{_await} && $_Q->{_asem} && $_Q->_pending() <= $_Q->{_tsem}) {
-            for (1 .. $_Q->{_asem}) { 1 until syswrite $_Q->{_aw_sock}, $LF }
+            for (1 .. $_Q->{_asem}) { syswrite $_Q->{_aw_sock}, $LF }
             $_Q->{_asem} = 0;
          }
 
@@ -869,7 +871,7 @@ sub _heap_insert_high {
          }
 
          if ($_Q->{_await} && $_Q->{_asem} && $_Q->_pending() <= $_Q->{_tsem}) {
-            for (1 .. $_Q->{_asem}) { 1 until syswrite $_Q->{_aw_sock}, $LF }
+            for (1 .. $_Q->{_asem}) { syswrite $_Q->{_aw_sock}, $LF }
             $_Q->{_asem} = 0;
          }
 
@@ -901,7 +903,7 @@ sub _heap_insert_high {
          $_Q = $_all->{$_id};
 
          if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-            1 until syswrite $_Q->{_qw_sock}, $LF;
+            syswrite $_Q->{_qw_sock}, $LF;
          }
 
          if (chop $_buf) {
@@ -925,7 +927,7 @@ sub _heap_insert_high {
          $_Q = $_all->{$_id};
 
          if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-            1 until syswrite $_Q->{_qw_sock}, $LF;
+            syswrite $_Q->{_qw_sock}, $LF;
          }
 
          if (chop $_buf) {
@@ -1078,7 +1080,7 @@ sub _mce_m_clear {
       warn "Queue: (clear) is not allowed for fast => 1\n";
    }
    else {
-      if ($_Q->_has_data()) { 1 until sysread $_Q->{_qr_sock}, $_next, 1 }
+      if ($_Q->_has_data()) { sysread $_Q->{_qr_sock}, $_next, 1 }
       $_Q->_clear();
    }
 
@@ -1091,7 +1093,7 @@ sub _mce_m_enqueue {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
    push @{ $_Q->{_datq} }, @_;
 
@@ -1107,7 +1109,7 @@ sub _mce_m_enqueuep {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
    $_Q->_enqueuep($_p, @_);
 
@@ -1120,7 +1122,7 @@ sub _mce_m_dequeue {
    my ($_Q, $_cnt) = @_;
    my (@_items, $_buf, $_next, $_pending);
 
-   1 until sysread $_Q->{_qr_sock}, $_next, 1;  # block
+   sysread $_Q->{_qr_sock}, $_next, 1;  # block
 
    if (defined $_cnt && $_cnt ne '1') {
       @_items = $_Q->_dequeue($_cnt);
@@ -1135,7 +1137,7 @@ sub _mce_m_dequeue {
          $_pending = int($_pending / $_cnt) if (defined $_cnt);
          if ($_pending) {
             $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
-            for (1 .. $_pending) { 1 until syswrite $_Q->{_qw_sock}, $LF }
+            for (1 .. $_pending) { syswrite $_Q->{_qw_sock}, $LF }
          }
          $_Q->{_dsem} = $_pending;
       }
@@ -1145,7 +1147,7 @@ sub _mce_m_dequeue {
    }
    else {
       ## Otherwise, never to exceed one byte in the channel
-      if ($_Q->_has_data()) { 1 until syswrite $_Q->{_qw_sock}, $LF }
+      if ($_Q->_has_data()) { syswrite $_Q->{_qw_sock}, $LF }
    }
 
    $_Q->{_nb_flag} = 0;
@@ -1179,7 +1181,7 @@ sub _mce_m_insert {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
 
    $_Q->_insert($_i, @_);
@@ -1198,7 +1200,7 @@ sub _mce_m_insertp {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
    $_Q->_insertp($_p, $_i, @_);
 
@@ -1217,6 +1219,9 @@ sub _mce_m_insertp {
       $_dat_ex, $_dat_un, $_len, $_next, $_pending, $_tag
    );
 
+   my $_is_MSWin32 = ($^O eq 'MSWin32') ? 1 : 0;
+   my $_rdy = \&MCE::Util::_sock_ready;
+
    my $_req1 = sub {
       local $\ = undef if (defined $\);
 
@@ -1262,8 +1267,8 @@ sub _mce_m_insertp {
       $_lock_chn   = $_MCE->{_lock_chn};
 
       if ($_lock_chn) {
-         $_dat_ex = sub { 1 until sysread(  $_DAT_LOCK->{_r_sock}, my $_b, 1 ) };
-         $_dat_un = sub { 1 until syswrite( $_DAT_LOCK->{_w_sock}, '0' ) };
+         $_dat_ex = sub {  sysread ( $_DAT_LOCK->{_r_sock}, my $_b, 1 ) };
+         $_dat_un = sub { syswrite ( $_DAT_LOCK->{_w_sock}, '0' ) };
       }
 
       $_all = {};
@@ -1306,9 +1311,12 @@ sub _mce_m_insertp {
       $_dat_ex->() if $_lock_chn;
       print {$_DAT_W_SOCK} OUTPUT_W_QUE.$LF . $_chn.$LF;
       print {$_DAU_W_SOCK} $_Q->{_id}.$LF . $_t.$LF;
+      <$_DAU_W_SOCK>;
+
       $_dat_un->() if $_lock_chn;
 
-      1 until sysread $_Q->{_ar_sock}, $_next, 1;  # block
+      $_rdy->($_Q->{_ar_sock}) if $_is_MSWin32;
+      sysread $_Q->{_ar_sock}, $_next, 1;  # block
 
       return;
    }
@@ -1403,7 +1411,8 @@ sub _mce_m_insertp {
          $_cnt = 1;
       }
 
-      1 until sysread $_Q->{_qr_sock}, $_next, 1;  # block
+      $_rdy->($_Q->{_qr_sock}) if $_is_MSWin32;
+      sysread $_Q->{_qr_sock}, $_next, 1;  # block
 
       $_req2->(OUTPUT_D_QUE, $_Q->{_id}.$LF . $_cnt.$LF, $_cnt);
    }
@@ -1629,7 +1638,7 @@ MCE::Queue - Hybrid (normal and priority) queues
 
 =head1 VERSION
 
-This document describes MCE::Queue version 1.800
+This document describes MCE::Queue version 1.801
 
 =head1 SYNOPSIS
 
